import {go, newChannel, close} from 'js-go-channels'
import {_checkGenerator, _LinkedListBuffer} from 'js-go-channels'

const takeRequest = 'reduxTake'
const take = actionType => ({
  type: takeRequest,
  payload: {actionType}
})
const slurp = (takeRequests, iterator, nextValue) => {
  const {value, done} = iterator.next(nextValue)
  if (done) {
    return
  }
  if (value.type !== takeRequest) {
    throw new Error(`Unsupported request ${value}`)
  }
  const {payload: {actionType}} = value
  // save in takeRequests
  takeRequests[actionType] = takeRequests[actionType] || new _LinkedListBuffer()
  takeRequests[actionType].add({iterator})
}

export function createMiddleware() {
  let dispatch = () => { throw new Error('unsupported') }
  let getState = () => { throw new Error('unsupported') }
  const redux = newChannel()
  const takeRequests = {}
  go(function* () {
    const action = yield redux.take()
    const pending = takeRequests[action.type]
    if (pending) {
      const {iterator} = pending.pop() || {}
      if (iterator) {
        slurp(takeRequests, iterator, action)
      }
    }
  })
  return {
    register(generatorFn) {
      const generator = generatorFn({dispatch, getState, take})
      const iterator = _checkGenerator(generator)
      slurp(takeRequests, iterator)
    },
  
    middleware({dispatch: d, getState: g}) {
      dispatch = d
      getState = g
      return next => action => {
        redux.asyncPut(action)
        return next(action)
      }
    },
  }
}
